@@ -183,8 +183,6 @@ static void        contact_list_view_row_expand_or_collapse_cb (EmpathyContactLi
 								gpointer                    user_data);
 static void        contact_list_view_action_cb                 (GtkAction                  *action,
 								EmpathyContactListView      *view);
-static void        contact_list_view_action_activated          (EmpathyContactListView      *view,
-								EmpathyContact              *contact);
 static void        contact_list_view_voip_activated            (EmpathyContactListView      *view,
 								EmpathyContact              *contact);
 
@@ -263,6 +261,7 @@ static const gchar *ui_info =
 	"  </popup>"
 	"  <popup name='Group'>"
 	"    <menuitem action='Rename'/>"
+	"    <menuitem action='Remove'/>"
 	"  </popup>"
 	"</ui>";
 
@@ -1034,27 +1033,21 @@ contact_list_view_cell_set_background (EmpathyContactListView *view,
 
 	style = gtk_widget_get_style (GTK_WIDGET (view));
 
-	if (!is_group) {
-		if (is_active) {
-			color = style->bg[GTK_STATE_SELECTED];
+	if (!is_group && is_active) {
+		color = style->bg[GTK_STATE_SELECTED];
 
-			/* Here we take the current theme colour and add it to
-			 * the colour for white and average the two. This
-			 * gives a colour which is inline with the theme but
-			 * slightly whiter.
-			 */
-			color.red = (color.red + (style->white).red) / 2;
-			color.green = (color.green + (style->white).green) / 2;
-			color.blue = (color.blue + (style->white).blue) / 2;
+		/* Here we take the current theme colour and add it to
+		 * the colour for white and average the two. This
+		 * gives a colour which is inline with the theme but
+		 * slightly whiter.
+		 */
+		color.red = (color.red + (style->white).red) / 2;
+		color.green = (color.green + (style->white).green) / 2;
+		color.blue = (color.blue + (style->white).blue) / 2;
 
-			g_object_set (cell,
-				      "cell-background-gdk", &color,
-				      NULL);
-		} else {
-			g_object_set (cell,
-				      "cell-background-gdk", NULL,
-				      NULL);
-		}
+		g_object_set (cell,
+			      "cell-background-gdk", &color,
+			      NULL);
 	} else {
 		g_object_set (cell,
 			      "cell-background-gdk", NULL,
@@ -1320,7 +1313,7 @@ contact_list_view_row_activated_cb (EmpathyContactListView *view,
 			    -1);
 
 	if (contact) {
-		contact_list_view_action_activated (view, contact);
+		empathy_chat_with_contact (contact);
 		g_object_unref (contact);
 	}
 }
@@ -1403,7 +1396,7 @@ contact_list_view_action_cb (GtkAction             *action,
 	parent = empathy_get_toplevel_window (GTK_WIDGET (view));
 
 	if (contact && strcmp (name, "Chat") == 0) {
-		contact_list_view_action_activated (view, contact);
+		empathy_chat_with_contact (contact);
 	}
 	else if (contact && strcmp (name, "Call") == 0) {
 		contact_list_view_voip_activated (view, contact);
@@ -1434,6 +1427,12 @@ contact_list_view_action_cb (GtkAction             *action,
 	}
 	else if (group && strcmp (name, "Rename") == 0) {
 	}
+	else if (group && strcmp (name, "Remove") == 0) {
+		EmpathyContactList *list;
+
+		list = empathy_contact_list_store_get_list_iface (priv->store);
+		empathy_contact_list_remove_group (list, group);
+	}
 
 	g_free (group);
 	if (contact) {
@@ -1441,22 +1440,6 @@ contact_list_view_action_cb (GtkAction             *action,
 	}
 }
 
-static void
-contact_list_view_action_activated (EmpathyContactListView *view,
-				    EmpathyContact         *contact)
-{
-	MissionControl *mc;
-
-	mc = empathy_mission_control_new ();
-	mission_control_request_channel (mc,
-					 empathy_contact_get_account (contact),
-					 TP_IFACE_CHANNEL_TYPE_TEXT,
-					 empathy_contact_get_handle (contact),
-					 TP_HANDLE_TYPE_CONTACT,
-					 NULL, NULL);
-	g_object_unref (mc);
-}
-
 static void
 contact_list_view_voip_activated (EmpathyContactListView *view,
 				  EmpathyContact         *contact)
